package edu.kit.ipd.descartes.mm.applicationlevel.repository.compositediagram.popup.actions;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collections;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain.Registry;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.graphiti.dt.IDiagramTypeProvider;
import org.eclipse.graphiti.features.context.impl.AddContext;
import org.eclipse.graphiti.mm.pictograms.Diagram;
import org.eclipse.graphiti.mm.pictograms.PictogramsFactory;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.services.IPeService;
import org.eclipse.graphiti.ui.services.GraphitiUi;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipselabs.spray.runtime.graphiti.wizard.FileOpener;
import org.eclipselabs.spray.runtime.graphiti.wizard.IDiagramInitializer;
import org.eclipselabs.spray.runtime.graphiti.wizard.NewDiagramPage;

import com.google.inject.Inject;
import com.google.inject.name.Named;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.CompositeComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;

public class InitCreateDiagramWizard implements IObjectActionDelegate {

	private Shell shell;
	private IStructuredSelection selection;
	private URI domainModelURI;
	private String fileName;
	
	@Inject
    private NewDiagramPage      newDiagramPage;
    @Inject
    @Named("diagramTypeId")
    private String              diagramTypeId;
    @Inject
    private IDiagramInitializer diagramInitializer;
    @Inject
    private FileOpener          fileOpener;
    @Inject
    private IWorkspace          workspace;
	
	/**
	 * Constructor for Action1.
	 */
	public InitCreateDiagramWizard() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		/*
		 * Load EObject root (=diagramRoot) from Resource (hopefully it contains type 'Repository')
		 */
    	TransactionalEditingDomain editingDomain = TransactionalEditingDomain.Factory.INSTANCE.createEditingDomain();
		ResourceSet resourceSet = editingDomain.getResourceSet();
		EObject diagramRoot = null;
		try {
			Resource resource = resourceSet.getResource(domainModelURI, true);
			diagramRoot = (EObject) resource.getContents().get(0);
		} catch (WrappedException e) {
			MessageDialog.openError(shell, "Wrong resource", "The selected Resource is false");
			return;
		}
		if (diagramRoot == null) {
			MessageDialog.openError(shell, "Diagram Root Element is null", "The root element in the resource is 'null'");
			return;
		}
		
		
		/*
		 * Load Repository from diagramRoot
		 */
		TreeIterator<EObject> it = diagramRoot.eAllContents();
		final Repository model;
		if (diagramRoot instanceof Repository) {
			model = (Repository) diagramRoot;
		} else {
			model = null;
		}
		
		//else {
//			//TODO: If  >1 Repository exists: present them to choose one
//			while (it.hasNext()) {
//				EObject possibleRepo = it.next();
//				if (possibleRepo instanceof Repository) {
//					model = (Repository) possibleRepo;
//					break;
//				}
//			}
//		}
		
		/*
		 * Testweises laden von EOBjects to add
		 */
		CompositeComponent testObj = null;
		while (it.hasNext()) {
			EObject possibleCC = it.next();
			if (possibleCC instanceof CompositeComponent) { //load CompositeComponent
				testObj = (CompositeComponent) possibleCC;
				break;
			}
		}
		if (testObj == null) {
			MessageDialog.openError(shell, "No CompositeComponent found", "Resource does not contain a 'CompositeComponent' EObject");
			return;
		}
		
		
		/*
		 * If no model found exit
		 */
		if (model == null) {
			MessageDialog.openError(shell, "No Repository found", "Resource does not contain a 'Repository' EObject");
			return;
		}
		
		/*
		 * Create empty diagram
		 */
		final String diagPath = domainModelURI.toPlatformString(true).replaceFirst(".repository", ".diagram");
		final ResourceSet rs = new ResourceSetImpl();
		final URI uri = URI.createPlatformResourceURI(diagPath, true);
		final Resource resource = rs.createResource(uri);
//		Diagram diagram = Graphiti.getPeCreateService().createDiagram(diagramTypeId, fileName, true);
		Diagram diagram = PictogramsFactory.eINSTANCE.createDiagram();
		diagram.setDiagramTypeId(diagramTypeId);
		diagram.setName(fileName);
		diagramInitializer.initialize(diagram);
		
		
		/*
		 * TODO Add Repository/EObject to diagram and link
		 */
		String dtId = GraphitiUi.getExtensionManager().getDiagramTypeProviderId(diagramTypeId);
		IDiagramTypeProvider dtp = GraphitiUi.getExtensionManager().createDiagramTypeProvider(diagram, dtId);
		AddContext ac = new AddContext();
		ac.setNewObject(testObj);
		ac.setTargetContainer(diagram);
		dtp.getFeatureProvider().addIfPossible(ac);
		
		
		/*
		 * Add Diagram to resource file
		 */
		resource.getContents().add(diagram);
		
//		TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(diagram);
//		domain.getCommandStack().execute(new RecordingCommand(domain) {
//		   public void doExecute() {
//			   diagram.eResource().getContents().add(model);
//		   }
//		});	   
			   
		/*
		 * Save resource and open editor
		 */
		//org.eclipselabs.spray.runtime.graphiti.wizard.NewDiagramWizard nutzt das folgende:
		//public void run(IProgressMonitor monitor) throws InvocationTargetException 
		try {
			resource.save(Collections.emptyMap());
		} catch (IOException e) {
			e.printStackTrace();
		}
		IFile file = workspace.getRoot().getFile(new Path(uri.toPlatformString(true)));
		fileOpener.selectAndReveal(file);
		fileOpener.openFileToEdit(shell, file);
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		this.domainModelURI = null;
		action.setEnabled(false);
		if (!(selection instanceof IStructuredSelection) || selection.isEmpty()) {
			action.setEnabled(true);
			return;
		}
//		this.selection = (IStructuredSelection) selection;
//		IFile file = (IFile) this.selection.getFirstElement();
		IFile file = (IFile) ((IStructuredSelection) selection).getFirstElement();
		this.fileName = file.getName().replaceAll(".repository", "");
		this.domainModelURI = URI.createPlatformResourceURI(file.getFullPath().toString(), true);
		action.setEnabled(true);
	}

}
